##
# yATF_ES
#  mrYoung Anti-TeamFlash (EventScripts)
#
# by Baptiste Boilet a.k.a. mRyOuNg
#  inspired by Medda and his wonderful script Medda_ATFS
#
# Requirements
#  EventScripts 2.0 or later
#  ES Tools 0.417+ or later
##

## libs imports
import es
import playerlib
import langlib
import gamethread

## addon info decl
info = es.AddonInfo()
info.name     = "yATF_ES - mrYoung Anti-TeamFlash ES"
info.version  = "v0.5b3"
info.url      = "http://addons.eventscripts.com/addons/view/yatf_es"
info.basename = "yatf_es"
info.author   = "mRyOuNg"

## cvar decl
yatf_es_ver             = es.ServerVar("yatf_es_ver"            , info.version, "yatf_es version")
yatf_es_enable          = es.ServerVar("yatf_es_enable"         , "1"         , "yatf_es enable/disable")
yatf_es_warmup          = es.ServerVar("yatf_es_warmup"         , "90"        , "yatf_es warmup duration (in seconds)")
yatf_es_notifyteamonly  = es.ServerVar("yatf_es_notifyteamonly" , "1"         , "yatf_es enable/disable notify team only")
yatf_es_flashduration   = es.ServerVar("yatf_es_flashduration"  , "1.00"      , "yatf_es flash duration (seconds)")
yatf_es_spawnprotection = es.ServerVar("yatf_es_spawnprotection", "5"         , "yatf_es spawn protection duration (in seconds)")
yatf_es_rsroundscount   = es.ServerVar("yatf_es_rsroundscount"  , "3"         , "yatf_es rounds count of restriction for spawn tf")
yatf_es_restrictpercent = es.ServerVar("yatf_es_restrictpercent", "30"        , "yatf_es percent of flashed players to trigger a restriction")
yatf_es_rproundscount   = es.ServerVar("yatf_es_rproundscount"  , "1"         , "yatf_es rounds count of restriction for percent tf")
yatf_es_restrictcumul   = es.ServerVar("yatf_es_restrictcumul"  , "5"         , "yatf_es cumulative count of flashed players to trigger a restriction")
yatf_es_rcroundscount   = es.ServerVar("yatf_es_rcroundscount"  , "2"         , "yatf_es rounds count of restriction for cumulative tf")

## global var decl
# text will helps in multi-languages management
g_text = None
# spawnprotection vars helps in management of the spawn protection feature
g_spawnprotectionstatus = 0
g_spawnprotectionduration = 0
# warmup vars helps in management of the warmup feature
g_warmup_previousyatfstatus = 0
# pinfo dict will contains informations about connected players
#  info about a player is a list: [numberOfRestrictedRoundsLeft, numberOfFlashedTeamates]
g_pinfo = {}
# tfinfo dict will contains informations about team flashed players
#  info about a player is a list: [playersTeam]
g_tfinfo = {}

## load block
# called at script loads
def load():
	# global vars decl.
	global g_text, g_spawnprotectionduration
	# making yatf_es_ver public for convenience
	yatf_es_ver.makepublic()
	# loading configuration
	es.mexec("../addons/eventscripts/yatf_es/yatf_es.cfg")
	# loading multi-languages strings
	g_text = langlib.Strings(es.getAddonPath("yatf_es") + "/strings.ini")
	# computing real spawnprotection time
	g_spawnprotectionduration = int(es.ServerVar("mp_freezetime")) + int(yatf_es_spawnprotection)

## es_map_start event
# triggered on map starts
def es_map_start(event_var):
	# global vars decl.
	global g_spawnprotectionduration, g_warmup_previousyatfstatus
	# re-loading configuration
	es.mexec("../addons/eventscripts/yatf_es/yatf_es.cfg")
	# clearing g_tfinfo (just to be sure)
	g_tfinfo.clear()
	# computing real spawnprotection time
	g_spawnprotectionduration = int(es.ServerVar("mp_freezetime")) + int(yatf_es_spawnprotection)
	# warmup mgmt
	if int(yatf_es_warmup) > 0:
		# if warmup feature is enabled
		g_warmup_previousyatfstatus = str(yatf_es_enable)
		es.server.queuecmd("yatf_es_enable 0")
		gamethread.delayed(int(yatf_es_warmup), yatf_es_warmupmgmt)

## round_start event
# triggered on round_start
def round_start(event_var):
	# global vars decl.
	global g_spawnprotectionstatus, g_pinfo
	# clearing g_tfinfo (just to be sure)
	g_tfinfo.clear()
	# unrestrict flashbang for all players just to be suer
	es.server.queuecmd("est_UnRestrict #a flashbang")
	if int(yatf_es_enable) != 0:
		# if yatf is enabled
		# spawn protection management
		g_spawnprotectionstatus = 1
		gamethread.delayed(g_spawnprotectionduration, yatf_es_fspawnpmgmt)
		# flashbang restrict mgmt
		l_restricttokens = {}
		for l_key,l_value in g_pinfo.iteritems():
			if l_value[0] > 0:
				l_restricttokens['roundcount'] = str(l_value[0])
				l_curplayer = playerlib.getPlayer(l_key)
				es.server.queuecmd("est_Restrict %s flashbang" % (l_key))
				es.tell(int(l_curplayer), "#multi", "#greenyATF_ES. " + g_text("yatf_es restrict_notice", l_restricttokens, l_curplayer.get("lang")))
				g_pinfo[l_key][0] = g_pinfo[l_key][0] - 1

## player_connect event
# triggered when a player connects
def player_connect(event_var):
	# global vars decl.
	global g_pinfo
	# adding a new element in pinfo dict
	g_pinfo[event_var['userid']] = [0, 0]

## player_disconnect event
# triggered when a player disconnects
def player_disconnect(event_var):
	# global vars decl.
	global g_pinfo
	# removing associated element in pinfo dict
	del g_pinfo[event_var['userid']]

## player_blind event
# triggered when a player is blinded
def player_blind(event_var):
	# global vars decl.
	global g_tfinfo
	# if yatf enabled
	if int(yatf_es_enable) != 0:
		# if flashed player is alive, and flashduration is above the limit, it's a real flash
		l_curflashed = playerlib.getPlayer(event_var['userid'])
		l_curflashedisdead = l_curflashed.get("isdead")
		l_curflashedflashduration = es.getplayerprop(event_var['userid'], "CCSPlayer.m_flFlashDuration")
		if int(l_curflashedisdead) == 0 and float(l_curflashedflashduration) >= float(yatf_es_flashduration):
			# storing flashed player in tfinfo
			g_tfinfo[event_var['userid']] = event_var['es_userteam']

## flashbang_detonate event
# triggered when a flashbang detonates
# note: this event occurs right after player_blind event
def flashbang_detonate(event_var):
	# global vars decl.
	global g_pinfo
	# if yatf enabled
	if int(yatf_es_enable) != 0:
		# misc. vars init.
		l_tfcount = 0 # will contains teamflashs count
		l_tfplayersname = [] # will contains flashed players name
		# for each "flashed" players, we must check if they are teamflashed
		for l_key,l_value in g_tfinfo.iteritems():
			# if it's not a "self-flashing"
			if event_var['userid'] != l_key:
				# if flashed player is in the same team as flasher player
				if event_var['es_userteam'] == l_value: # means, that's a team flash
					l_tfcount = l_tfcount + 1 # incrementing tf_count
					l_curflashed = playerlib.getPlayer(l_key)
					l_tfplayersname.append(l_curflashed.get("name"))
		# if there was at least a team flash
		if l_tfcount > 0:
			# misc. vars init.
			l_tftokens = {}
			l_tftokens['flasher'] = event_var['es_username']
			l_tftokens['tfcount'] = str(l_tfcount)
			l_notifiedplayers = []
			# teamflashs count
			g_pinfo[event_var['userid']][1] = g_pinfo[event_var['userid']][1] + l_tfcount
			# spawn protection management
			if g_spawnprotectionstatus != 0:
				es.server.queuecmd("es_sexec " + str(event_var['userid']) + " kill")
				# flashbang restriction for spawn tf mgmt
				if int(yatf_es_rsroundscount) > 0:
					g_pinfo[event_var['userid']][0] = g_pinfo[event_var['userid']][0] + int(yatf_es_rsroundscount)
			# time to notify players
			if int(yatf_es_notifyteamonly) == 0:
				# if notifyteamonly is not enabled
				l_notifiedplayers = playerlib.getPlayerList("#all")
			else:
				# notifyteamonly is enabled
				if int(event_var['es_userteam']) == 2:
					# if it's #t team
					l_notifiedplayers = playerlib.getPlayerList("#t")
				elif int(event_var['es_userteam']) == 3:
					# if it's #ct team
					l_notifiedplayers = playerlib.getPlayerList("#ct")
			# for each player to notify, let's display the message
			for l_curnotifplayer in l_notifiedplayers:
				# for each flashed player name
				for l_curtfplayername in l_tfplayersname:
					l_tftokens['flashed'] = l_curtfplayername
					es.tell(int(l_curnotifplayer), "#multi", "#greenyATF_ES. " + g_text("yatf_es tf_notice", l_tftokens, l_curnotifplayer.get("lang")))
				es.tell(int(l_curnotifplayer), "#multi", "#greenyATF_ES. " + g_text("yatf_es tfcount_notice", l_tftokens, l_curnotifplayer.get("lang")))
				if g_spawnprotectionstatus != 0:
					es.tell(int(l_curnotifplayer), "#multi", "#greenyATF_ES. " + g_text("yatf_es tf_spawn_notice", l_tftokens, l_curnotifplayer.get("lang")))
			# percent tf restriction
			if int(yatf_es_rproundscount) > 0: # if restrict percent is enabled
				l_playerscount = es.getplayercount(int(event_var['es_userteam']))
				if int(l_playerscount) != -1: # if getplayercount doesn't returns error
					# computing percent of players flashed
					l_percentflashed = (float(l_tfcount) / float(l_playerscount)) * 100.0
					if l_percentflashed >= float(yatf_es_restrictpercent):
						g_pinfo[event_var['userid']][0] = g_pinfo[event_var['userid']][0] + int(yatf_es_rproundscount)
			# cumulative tf restriction
			if int(yatf_es_rcroundscount) > 0: # if restrict cumulative is enabled
				while g_pinfo[event_var['userid']][1] >= int(yatf_es_restrictcumul):
					g_pinfo[event_var['userid']][0] = g_pinfo[event_var['userid']][0] + int(yatf_es_rcroundscount)
					g_pinfo[event_var['userid']][1] = g_pinfo[event_var['userid']][1] - int(yatf_es_restrictcumul)
	g_tfinfo.clear()

## yatf_es_fspawnpmgmt function
# called to manage gamethread delayed spawn protection management
def yatf_es_fspawnpmgmt():
	# global vars decl.
	global g_spawnprotectionstatus
	# spawn protection mgmt
	g_spawnprotectionstatus = 0

## yatf_es_warmupmgmt function
# called to manage gamethread delayed warmup management
def yatf_es_warmupmgmt():
	# warmup mgmt
	es.server.queuecmd("yatf_es_enable %s" % (str(g_warmup_previousyatfstatus)))
